unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, unit2;

type
  TForm1 = class(TForm)
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
    updaters:THTMLTagURLUpdaterList;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

uses DIUnicode, DIHtmlParser, DIHtmlMisc, DIHtmlWriterPlugin, DIUnicodeHtmlWriter;

function updatePage(page, pagePath, newPath, domain, oldDomain:string; moved:TStrings; updaters:THTMLTagURLUpdaterList; ahtmlParser:TDIHtmlParser=nil):string;
var htmlParser:TDIHtmlParser;
    htmlWriter:TDIHtmlWriterPlugin;
begin
  ensureCorrectPath(pagePath);
  ensureCorrectPath(newPath);
  assert(isDomain(domain));
  assert(isDomain(oldDomain));

  if ahtmlParser=nil then htmlParser:=TDIHtmlParser.Create(nil)
                     else htmlParser:=ahtmlParser;

  htmlWriter:=TDIHtmlWriterPlugin.Create(nil);
  htmlWriter.FilterText:=fishow;
  htmlWriter.FilterHtmlTags.SetStartEnd(fiShowHidden);

//  htmlWriter.Enabled:=true;

  htmlParser.InsertPluginFirst(htmlWriter);
  htmlWriter.Writer.DataStream:=TStringStream.Create('');
  if DecodingEntitiesCount = 0 then
  begin
    RegisterHtmlDecodingEntities;
    RegisterHtmlEncodingEntities;
  end;

  htmlparser.SetAllFilters(fiHide);
  htmlparser.FilterHtmlTags.StartTags:=fiShow;
  htmlparser.FilterHtmlTags.EndTags:=fiShow;
  htmlparser.FilterCDataSections:=fiShow;
  htmlparser.FilterComments:=fiShow;
  htmlparser.FilterScripts:=fiShow;
  htmlparser.FilterStyles:=fiShow;
//  htmlparser.Filter
  RegisterHtmlTags;
  htmlparser.SourceBufferAsStrA:=page;
             
  with htmlparser do 
    while ParseNextPiece do
    begin
      assert(htmltag.tagid<>tag_base_id);// this will render the algorithm useless since it enforces a default on relative paths
      // if this is found, another approach must be used. I haven't seen this ever, so I am not implementing this behaviour

      {$Message warn 'if tag is form and action is '' then set action=current document path+name (and maybe change domain: the tricky part to whatch out for)'}
      if (PieceType=ptHtmlTag) and (htmltag.TagType=ttStartTag) then
        updaters.updatetag(HtmlTag, pagePath, newPath, domain, oldDomain, moved);
      htmlwriter.WriteCurrentPiece;
{      case PieceType of
        ptHtmlTag:htmlWriter.WriteCurrentHtmlTag;
        ptComment:htmlwriter.WriteCurrentComment;
        else
      end;{}
    end;

  htmlWriter.Writer.FlushDataBuffer;

  result:=TStringStream(htmlWriter.Writer.DataStream).DataString;

  htmlWriter.Writer.FreeDataStream;

  htmlWriter.free;
  
  if ahtmlParser=nil then htmlParser.free;
end;

function stringFromFile(filename:string):string;
begin
  with tfilestream.create(filename, fmopenread) do
  begin
    setlength(result, size);
    Read(pansichar(result)^, size);    
    free;
  end;
end;

procedure stringToFile(str, filename:string);
begin
  with tfilestream.create(filename, fmcreate) do
  begin
    write(pansichar(str)^, length(str));
    free;
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
var moved:tstringlist;
begin
  updaters:=THTMLTagURLUpdaterList.Create;
  with updaters do
  begin
    add(TAG_A_ID, ['href']);
    add(TAG_APPLET_ID, ['code'{,'archive', 'codebase' might not be needed| also introduces different behaviour}]);
    add(TAG_AREA_ID, ['href']);
    add(TAG_BGSOUND_ID, ['src']);
    add(TAG_EMBED_ID, ['src']);
    add(TAG_FORM_ID, ['action']);// this one is tricky... going on default for now but watch out
    add(TAG_FRAME_ID, ['longdesc', 'src']);
    add(TAG_IFRAME_ID, ['longdesc', 'src']);
//    add(TAG_IMAGE_ID, ['']); ??
    add(TAG_IMG_ID, ['src', 'ismap', 'longdesc', 'usemap']);
    add(TAG_LINK_ID, ['href']);
    add(TAG_OBJECT_ID, ['data'{, 'archive', 'codebase', 'usemap'}]);
    add(TAG_SCRIPT_ID, ['src']);
  end;
  moved:=tstringlist.create;
  // the following list will probably need to be generated or created by hand
  moved.add('/testdir/page2.html=/newpath/');// all moved paths must be absolute !!!
  moved.add('/testdir/page3.html=/newpath/');// all moved paths must be absolute !!!
  moved.add('/some/other/dir/pagex.html=/some/other/moved/dir/pagex.html');// all moved paths must be absolute !!!
  // some moved paths can be calculated, relative to the moved path, but some other cannot be calculated because no information
  // is available to where the path was moved, like in the 3rd case above. se that is why we need to add this information as well

  stringtofile(updatepage(stringfromfile('test.html'), '/testdir/', '/newpath/', 'www.testdomain.com', 'www.original-domain.com', moved, updaters), 'result.html');
  // test.html was in /testdir/ directory on original-domain.com
  // full url was: http://[www.]original-domain.com/testdir/test.html
  // it will be moved to http://[www.]testdomain.com/newpath/test.html
  // all relative paths are changed to absolute paths
  // if an url will be present in the moved paths, it is considered moved to new location and updated accordingly, otherwise, it is left with the old domain

  moved.free;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  updaters.Free;
end;

end.
