unit Unit2;

interface

uses DIHtmlParser, contnrs, classes;

type
  TStringArray=array of string;

  THTMLTagURLUpdater=class
  private
    FTagId:cardinal;
    FAttributes:TStringArray;
  public
    property TagID:cardinal read FTagId;
    procedure update(htmlTag:TDiHtmlTag; pagePath, newPath, domain, oldDomain:string; moved:TStrings); virtual;
    constructor Create(ATagId:cardinal; AAttributes:TStringArray);
  end;

  THTMLTagURLUpdaterList=class
  private
    updaters:TObjectList;
  public
    constructor Create;
    destructor Destroy; override;
    procedure add(ATagId:cardinal; AAttributes:array of string); overload;
    procedure add(updater:THTMLTagURLUpdater); overload;
    procedure updatetag(htmlTag:TDiHtmlTag; pagePath, newPath, domain, oldDomain:string; moved:TStrings);
  end;

function isDomain(domain:string):boolean;
procedure ensureCorrectPath(var path:string);

implementation

uses sysutils, iduri, winsock;

type TCharSet=set of char;

{$I 'compiler_versions.inc'} 

function allCharsInSet(str:string; aset:TCharSet):boolean;
var i:integer;
begin
  i:=1;
  while (i<=length(str)) and (str[i] in aset) do
    inc(i);
  result:=i>length(str);
end;

function isDomain(domain:string):boolean;
begin
  result:=(domain<>'') and (allcharsinset(domain, ['a'..'z','A'..'Z','0'..'9','.','-']));// should put some more validation here
end;

procedure ensureCorrectPath(var path:string);
begin
  assert((path<>'') and (path[1]='/'));
  if (path<>'') and (path[length(path)]<>'/') then
    path:=path+'/';
end;

{ THTMLTagURLUpdater }

constructor THTMLTagURLUpdater.Create(ATagId: cardinal;
  AAttributes: TStringArray);
begin
  FTagId:=ATagId;
  FAttributes:=AAttributes;
end;

//http://www.swissdelphicenter.ch/torry/showcode.php?id=170
function GetIPFromHost(const HostName: string): string;
type
  TaPInAddr = array[0..10] of PInAddr;
  PaPInAddr = ^TaPInAddr;
var
  phe: PHostEnt;
  pptr: PaPInAddr;
  i: Integer;
  GInitData: TWSAData;
begin
  WSAStartup($101, GInitData);
  Result := '';
  phe := GetHostByName(PChar(HostName));
  if phe = nil then Exit;
  pPtr := PaPInAddr(phe^.h_addr_list);
  i := 0;
  while pPtr^[i] <> nil do
  begin
    Result := inet_ntoa(pptr^[i]^);
    Inc(i);
  end;
  WSACleanup;
end;

function samehost(host1, host2:string):boolean;
begin
  result:=sametext(host1, host2) or (GetIPFromHost(host1)=GetIPFromHost(host2));
end;

procedure THTMLTagURLUpdater.update(htmlTag:TDiHtmlTag; pagePath, newPath, domain, oldDomain:string; moved:TStrings);
  function getAttributeValue(attributeName:string):string;
  var i:integer;
  begin
    i:=0;
    with HtmlTag do
    begin
      while (i<Count) and (not sametext(NameAt[i],attributeName)) do
        inc(i);
      if i<Count then result:=ValueAt[i]
                 else result:='';
    end;
  end;

  procedure setAttributeValue(attributeName, attributeValue:string);
  var i:integer;
  begin
    i:=0;
    with HtmlTag do
    begin
      while (i<Count) and (not sametext(NameAt[i],attributeName)) do
        inc(i);
      if i<Count then ValueAt[i]:=attributeValue
                 else raise Exception.create('Unkown attribute '+attributeName+' for '+htmltag.TagName);
    end;
  end;

  procedure makeFullPath(var path:string);
  begin
    if (path='') or (path[length(path)]<>'/') then
      path:=path+'/';
    if path[1]<>'/' then
      path:='/'+path;
  end;

  {$IFDEF DELPHI_6_DOWN}
  function valueFromIndex(list:TStrings; index:integer):string;
  begin
    if Index >= 0 then Result := Copy(list[Index], Length(list.Names[Index]) + 2, MaxInt)
                  else Result := '';
  end;
  {$ENDIF}

  function isMoved(path, doc:string; var mPath:string):boolean;
  var i:integer;
  begin
    result:=false;
    if moved.count=0 then
      exit;
    makeFullPath(path);
    doc:=path+doc;
    i:=0;
    while (i<moved.count) and (not sametext(doc, moved.Names[i])) do
      inc(i);
    result:=i<moved.count;
    if result then
    begin
{$IFDEF DELPHI_7_UP}
      mPath:=moved.ValueFromIndex[i];
{$ELSE}
      mPath:=valueFromIndex(moved, i);
{$ENDIF}
      makeFullpath(mPath);
    end;
  end;

  procedure breakUp(s:string; l:TStrings);
  var t:string;
  begin
    assert((s<>'') and (s[1]='/') and (s[length(s)]='/'));
    l.clear;
    delete(s,1,1);
    repeat
      t:='';
      while (s<>'') and (s[1]<>'/') do
      begin
        t:=t+s[1];
        delete(s,1,1);
      end;
      delete(s,1,1);
      if t<>'' then
        l.add(t);
    until s='';
  end;

  function makeQualifiedPath(p:string):string; overload;
  var i:integer;
      pl:TStringList;
  begin
    pl:=TStringList.create;
    breakup(p, pl);
    i:=1;// skip first one
    while (i<pl.count) do
    begin
      assert((i>0) or (pl[0]<>'..'));
      if (pl[i]='..') and (pl[i-1]<>'..') then
      begin
        pl.Delete(i);
        pl.delete(i-1);
        dec(i);
      end                                 else
        inc(i);
    end;
    result:='/';
    for i:=1 to pl.count do
      result:=result+pl[i-1]+'/';
    pl.free;
  end;

  function makeQualifiedPath(p, r:string):string; overload;
  begin
    assert((r<>'') and (r[1]='/'));
    if (p='') or (p[1]<>'/') then
    begin
      if r[length(r)]<>'/' then
        r:=r+'/';
      result:=makeQualifiedPath(r+p);
    end                        else
      result:=makeQualifiedPath(p);
  end;

var i:integer;
    s, absPath, movedPath:string;
    u:TIdUri;
    m:boolean;
begin
  assert(htmlTag.TagID=TagID);

  ensureCorrectPath(pagePath);
  ensureCorrectPath(newPath);
  assert(isDomain(domain));
  assert(isDomain(oldDomain));
  // the above is done in the main program too, but if this unit will be used in some other projects, better safe than sorry

  u:=TIdURI.Create;
  for i:=1 to length(FAttributes) do
  begin
    s:=getAttributeValue(FAttributes[i-1]);
    if s<>'' then
    begin
      u.URI:=s;
      if u.Protocol='' then
        u.Protocol:='http';
      absPath:=makeQualifiedPath(u.path, pagePath);
      m:=isMoved(absPath, u.Document, movedPath);
      if u.Host='' then
      begin
        if m then u.Host:=domain
             else u.Host:=oldDomain;
      end          else
        if m and samehost(u.Host, oldDomain) then
          u.Host:=domain;
      {u.Port:=inttostr(ThePort);}// this might be needed if the destination server is not on default 80/443 and port is not included in uri
      if m and sametext(u.Host, domain) then u.Path:=movedPath
                                        else u.Path:=absPath;
      if s<>u.uri then
        setAttributeValue(FAttributes[i-1], u.uri);
    end;
  end;
  u.Free;
end;

{ THTMLTagURLUpdaterList }

procedure THTMLTagURLUpdaterList.add(ATagId: cardinal;
  AAttributes: array of string);
var a:TStringArray;
    i:integer;
begin
  setlength(a, length(AAttributes));
  for i:=1 to length(a) do
    a[i-1]:=AAttributes[i-1];
  updaters.Add(THTMLTagURLUpdater.Create(ATagId, a));
end;

procedure THTMLTagURLUpdaterList.add(updater: THTMLTagURLUpdater);
begin
  updaters.add(updater);
end;

constructor THTMLTagURLUpdaterList.Create;
begin
  updaters:=TObjectList.Create(true);
end;

destructor THTMLTagURLUpdaterList.Destroy;
begin
  updaters.Free;
  inherited;
end;

procedure THTMLTagURLUpdaterList.updatetag(htmlTag: TDiHtmlTag; pagePath,
  newPath, domain, oldDomain: string; moved:TStrings);
var i:integer;
begin
  i:=0;
  while (i<updaters.Count) and (THTMLTagURLUpdater(updaters[i]).TagID<>htmltag.TagID) do
    inc(i);
  if i<updaters.Count then
    THTMLTagURLUpdater(updaters[i]).update(htmltag, pagePath, newPath, domain, oldDomain, moved);
end;

end.
